=== modified file 'NEWS'
--- NEWS	2009-06-17 05:05:28 +0000
+++ NEWS	2009-06-19 07:28:41 +0000
@@ -57,6 +57,10 @@
 API Changes
 ***********
 
+* ProgressTasks now prefer to talk direct to their ProgressView not to the
+  UIFactory. 
+  (Martin Pool)
+
 * Removed overspecific error class ``InvalidProgressBarType``.
   (Martin Pool)
 

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2009-06-19 06:21:13 +0000
+++ bzrlib/progress.py	2009-06-19 07:28:41 +0000
@@ -72,9 +72,16 @@
     will not necessarily respect all these fields.
     """
 
-    def __init__(self, parent_task=None, ui_factory=None):
+    def __init__(self, parent_task=None, ui_factory=None, progress_view=None):
         """Construct a new progress task.
 
+        :param parent_task: Enclosing ProgressTask or None.
+
+        :param progress_view: ProgressView to display this ProgressTask.
+
+        :param ui_factory: The UI factory that will display updates; 
+            deprecated in favor of passing progress_view directly.
+
         Normally you should not call this directly but rather through
         `ui_factory.nested_progress_bar`.
         """
@@ -83,7 +90,9 @@
         self.total_cnt = None
         self.current_cnt = None
         self.msg = ''
+        # TODO: deprecate passing ui_factory
         self.ui_factory = ui_factory
+        self.progress_view = progress_view
         self.show_pct = False
         self.show_spinner = True
         self.show_eta = False,
@@ -102,16 +111,23 @@
         self.current_cnt = current_cnt
         if total_cnt:
             self.total_cnt = total_cnt
-        self.ui_factory._progress_updated(self)
+        if self.progress_view:
+            self.progress_view.show_progress(self)
+        else:
+            self.ui_factory._progress_updated(self)
 
     def tick(self):
         self.update(self.msg)
 
     def finished(self):
-        self.ui_factory._progress_finished(self)
+        if self.progress_view:
+            self.progress_view.task_finished(self)
+        else:
+            self.ui_factory._progress_finished(self)
 
     def make_sub_task(self):
-        return ProgressTask(self, self.ui_factory)
+        return ProgressTask(self, ui_factory=self.ui_factory,
+            progress_view=self.progress_view)
 
     def _overall_completion_fraction(self, child_fraction=0.0):
         """Return fractional completion of this task and its parents
@@ -140,7 +156,10 @@
 
     def clear(self):
         # XXX: shouldn't be here; put it in mutter or the ui instead
-        self.ui_factory.clear_term()
+        if self.progress_view:
+            self.progress_view.clear()
+        else:
+            self.ui_factory.clear_term()
 
 
 @deprecated_function(deprecated_in((1, 16, 0)))

=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2009-06-10 03:56:49 +0000
+++ bzrlib/tests/test_progress.py	2009-06-19 07:28:41 +0000
@@ -14,14 +14,21 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+
 import os
 from StringIO import StringIO
 
 from bzrlib import errors
-from bzrlib.tests import TestCase
+from bzrlib.progress import (
+    ProgressTask,
+    )
 from bzrlib.symbol_versioning import (
     deprecated_in,
     )
+from bzrlib.tests import TestCase
+from bzrlib.ui.text import (
+    TextProgressView,
+    )
 
 
 class _TTYStringIO(StringIO):
@@ -36,3 +43,72 @@
 
     def isatty(self):
         return False
+
+
+class TestTextProgressView(TestCase):
+    """Tests for text display of progress bars.
+
+    These try to exercise the progressview independently of its construction,
+    which is arranged by the TextUIFactory.
+    """
+    # The ProgressTask now connects directly to the ProgressView, so we can
+    # check them independently of the factory or of the determination of what
+    # view to use.
+    
+    def make_view(self):
+        out = StringIO()
+        view = TextProgressView(out)
+        view._width = 80
+        return out, view
+    
+    def make_task(self, parent_task, view, msg, curr, total):
+        # would normally be done by UIFactory; is done here so that we don't
+        # have to have one.
+        task = ProgressTask(parent_task, progress_view=view)
+        task.msg = msg
+        task.current_cnt = curr
+        task.total_cnt = total
+        return task
+
+    def test_render_progress_easy(self):
+        """Just one task and one quarter done"""
+        out, view = self.make_view()
+        task = self.make_task(None, view, 'reticulating splines', 5, 20)
+        view.show_progress(task)
+        self.assertEqual(
+'\r[####/               ] reticulating splines 5/20                               \r'
+            , out.getvalue())
+
+    def test_render_progress_nested(self):
+        """Tasks proportionally contribute to overall progress"""
+        out, view = self.make_view()
+        task = self.make_task(None, view, 'reticulating splines', 0, 2)
+        task2 = self.make_task(task, view, 'stage2', 1, 2)
+        view.show_progress(task2)
+        # so we're in the first half of the main task, and half way through
+        # that
+        self.assertEqual(
+r'[####-               ] reticulating splines:stage2 1/2'
+            , view._render_line())
+        # if the nested task is complete, then we're all the way through the
+        # first half of the overall work
+        task2.update('stage2', 2, 2)
+        self.assertEqual(
+r'[#########\          ] reticulating splines:stage2 2/2'
+            , view._render_line())
+
+    def test_render_progress_sub_nested(self):
+        """Intermediate tasks don't mess up calculation."""
+        out, view = self.make_view()
+        task_a = ProgressTask(None, progress_view=view)
+        task_a.update('a', 0, 2)
+        task_b = ProgressTask(task_a, progress_view=view)
+        task_b.update('b')
+        task_c = ProgressTask(task_b, progress_view=view)
+        task_c.update('c', 1, 2)
+        # the top-level task is in its first half; the middle one has no
+        # progress indication, just a label; and the bottom one is half done,
+        # so the overall fraction is 1/4
+        self.assertEqual(
+            r'[####|               ] a:b:c 1/2'
+            , view._render_line())

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2009-06-17 05:16:48 +0000
+++ bzrlib/tests/test_ui.py	2009-06-19 07:28:41 +0000
@@ -25,6 +25,9 @@
 
 import bzrlib
 import bzrlib.errors as errors
+from bzrlib.progress import (
+    ProgressTask,
+    )
 from bzrlib.symbol_versioning import (
     deprecated_in,
     )
@@ -33,7 +36,10 @@
     TestUIFactory,
     StringIOWrapper,
     )
-from bzrlib.tests.test_progress import _TTYStringIO
+from bzrlib.tests.test_progress import (
+    _NonTTYStringIO,
+    _TTYStringIO,
+    )
 from bzrlib.ui import (
     CLIUIFactory,
     SilentUIFactory,
@@ -107,22 +113,35 @@
     def test_progress_construction(self):
         """TextUIFactory constructs the right progress view.
         """
+        err = _TTYStringIO()
+        os.environ['TERM'] = 'xterm'
         os.environ['BZR_PROGRESS_BAR'] = 'none'
-        self.assertIsInstance(TextUIFactory()._progress_view,
+        self.assertIsInstance(TextUIFactory(stderr=err)._progress_view,
             NullProgressView)
 
         os.environ['BZR_PROGRESS_BAR'] = 'text'
-        self.assertIsInstance(TextUIFactory()._progress_view,
+        self.assertIsInstance(TextUIFactory(stderr=err)._progress_view,
             TextProgressView)
 
         os.environ['BZR_PROGRESS_BAR'] = 'text'
-        self.assertIsInstance(TextUIFactory()._progress_view,
+        self.assertIsInstance(TextUIFactory(stderr=err)._progress_view,
             TextProgressView)
 
         del os.environ['BZR_PROGRESS_BAR']
-        self.assertIsInstance(TextUIFactory()._progress_view,
+        self.assertIsInstance(TextUIFactory(stderr=err)._progress_view,
             TextProgressView)
 
+        # if not a tty, no progress bars
+        self.assertIsInstance(
+            TextUIFactory(stderr=_NonTTYStringIO())._progress_view,
+            NullProgressView)
+
+        # if a tty but dumb, no progress bars
+        os.environ['TERM'] = 'dumb'
+        self.assertIsInstance(
+            TextUIFactory(stderr=_TTYStringIO())._progress_view,
+            NullProgressView)
+
     def test_progress_note(self):
         stderr = StringIO()
         stdout = StringIO()
@@ -142,14 +161,15 @@
             pb.finished()
 
     def test_progress_note_clears(self):
-        stderr = StringIO()
-        stdout = StringIO()
-        # The PQM redirects the output to a file, so it
-        # defaults to creating a Dots progress bar. we
-        # need to force it to believe we are a TTY
+        stderr = _TTYStringIO()
+        stdout = _TTYStringIO()
+        # so that we get a TextProgressBar
+        os.environ['TERM'] = 'xterm'
         ui_factory = TextUIFactory(
             stdin=StringIO(''),
             stdout=stdout, stderr=stderr)
+        self.assertIsInstance(ui_factory._progress_view,
+            TextProgressView)
         pb = ui_factory.nested_progress_bar()
         try:
             # Create a progress update that isn't throttled
@@ -222,6 +242,7 @@
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
         out = _TTYStringIO()
+        os.environ['TERM'] = 'xterm'
         factory = TextUIFactory(stdin=StringIO("yada\ny\n"), stdout=out, stderr=out)
         pb = factory.nested_progress_bar()
         pb.show_bar = False
@@ -295,61 +316,3 @@
             pb.finished()
 
 
-class TestTextProgressView(TestCase):
-    """Tests for text display of progress bars.
-    """
-    # XXX: These might be a bit easier to write if the rendering and
-    # state-maintaining parts of TextProgressView were more separate, and if
-    # the progress task called back directly to its own view not to the ui
-    # factory. -- mbp 20090312
-    
-    def _make_factory(self):
-        out = StringIO()
-        uif = TextUIFactory(stderr=out)
-        uif._progress_view._width = 80
-        return out, uif
-
-    def test_render_progress_easy(self):
-        """Just one task and one quarter done"""
-        out, uif = self._make_factory()
-        task = uif.nested_progress_bar()
-        task.update('reticulating splines', 5, 20)
-        self.assertEqual(
-'\r[####/               ] reticulating splines 5/20                               \r'
-            , out.getvalue())
-
-    def test_render_progress_nested(self):
-        """Tasks proportionally contribute to overall progress"""
-        out, uif = self._make_factory()
-        task = uif.nested_progress_bar()
-        task.update('reticulating splines', 0, 2)
-        task2 = uif.nested_progress_bar()
-        task2.update('stage2', 1, 2)
-        # so we're in the first half of the main task, and half way through
-        # that
-        self.assertEqual(
-r'[####\               ] reticulating splines:stage2 1/2'
-            , uif._progress_view._render_line())
-        # if the nested task is complete, then we're all the way through the
-        # first half of the overall work
-        task2.update('stage2', 2, 2)
-        self.assertEqual(
-r'[#########|          ] reticulating splines:stage2 2/2'
-            , uif._progress_view._render_line())
-
-    def test_render_progress_sub_nested(self):
-        """Intermediate tasks don't mess up calculation."""
-        out, uif = self._make_factory()
-        task_a = uif.nested_progress_bar()
-        task_a.update('a', 0, 2)
-        task_b = uif.nested_progress_bar()
-        task_b.update('b')
-        task_c = uif.nested_progress_bar()
-        task_c.update('c', 1, 2)
-        # the top-level task is in its first half; the middle one has no
-        # progress indication, just a label; and the bottom one is half done,
-        # so the overall fraction is 1/4
-        self.assertEqual(
-            r'[####|               ] a:b:c 1/2'
-            , uif._progress_view._render_line())
-

